;;; radical.el -- Han character input method with radical/strokes infomation.

;; Copyright (C) 1995 Electrotechnical Laboratory, JAPAN.

;; Author: Satoru TOMURA <tomura@etl.go.jp>

;; Keywords: radical, strokes, Han character

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; The code of this program is extracted from busyu.el which is in Mule.

;;; Code:

(require 'mini-menu)

(defvar radical-code nil)
(defvar radical-code-alist
  '(("jis" radical-jis radical-data-jis)
    ("cns" radical-cns radical-data-cns)
    ("gb"  radical-gb  radical-data-gb)))

(defun radical-input-jis () (interactive) (radical-input "jis"))
(defun radical-input-cns () (interactive) (radical-input "cns"))
(defun radical-input-gb  () (interactive) (radical-input "gb" ))
(defun strokes-input-jis () (interactive) (strokes-input "jis"))
(defun strokes-input-cns () (interactive) (strokes-input "cns"))
(defun strokes-input-gb  () (interactive) (strokes-input "gb" ))

(defun read-radical-code ()
  (interactive)
  (let* ((completion-ignore-case t)
	 (code (completing-read "Which code ? "
				radical-code-alist
				nil t)))
    (if (= (length code) 0)
	(error "Should specify one of %s"
	       (mapconcat 'car radical-code-alist ", ")))
    code))

(defun get-radical-data (code)
  (let ((radical-code-slot (assoc code radical-code-alist)))
    (if (not radical-code-slot)
	(error "Invalid radical code \"%s\". Specify it with C-u M-x radical-input." code))
    (require (nth 1 radical-code-slot))
    (symbol-value (nth 2 radical-code-slot))))

(defun radical-input (&optional code)
  (interactive (list (if (or current-prefix-arg (null radical-code))
			 (read-radical-code)
		       radical-code)))
  (setq radical-code code)
  (let* ((radical-data (get-radical-data code))
	 (radical-table (nth 0 radical-data))
	 (radical-strokes-alist (nth 1 radical-data))
	 (loop1 t)
	 (loop2)
	 (loop3)
	 (val1)
	 (val2)
	 (radicalname)
	 (nradical (1- (length radical-table)))
	 (stroke1)
	 (stroke2)
	 (query (nth 3 radical-data))
	 (q-chinese-char (nth 0 query))
	 (q-radical-stroke (nth 1 query))
	 (q-radical (nth 2 query))
	 (q-total-stroke (nth 3 query))
	 (q-stroke-suffix (nth 4 query)))
    (while (or loop1 loop2 loop3)
      (if loop1
	  (progn
	    (setq loop1 nil)
	    (setq stroke1
		  (string-to-int
		   (read-from-minibuffer
		    (format "%s (1-%d): " q-radical-stroke nradical)
		    (if stroke1 (int-to-string stroke1)))))
	    (and (< 0 stroke1) (<= stroke1 nradical) ; 92.9.30 by K.Mitani
		 (setq loop2 t))))
      (let ((inhibit-quit t))
	(if loop2
	    (progn
	      (setq loop2 nil)
	      (setq radicalname
		    (menu:select-from-menu
		     (list 'menu (concat q-radical ":") 
			   (aref radical-table stroke1))))
	      (if quit-flag
		  (progn
		    (setq quit-flag nil)
		    (setq loop1 t))
		(setq loop3 t)
		(setq val1 (cdr (assoc radicalname radical-strokes-alist))))))
	(if loop3
	    (progn
	      (setq loop3 nil)
	      (setq stroke2
		    (menu:select-from-menu
		     (list 'menu (concat q-total-stroke ":")
			   (cons (cons "*" val1)
				 (mapcar (function
					  (lambda (x)
					    (cons (format " %s%s" (car x) q-stroke-suffix)
						  (cdr x))))
					 val1)))))
	      (if quit-flag
		  (progn
		    (setq quit-flag nil)
		    (setq loop2 t))
		(setq val2
		      (menu:select-from-menu 
		       (list 'menu (concat q-chinese-char ":")
			     (if (stringp stroke2)
				 (radical-break-string stroke2)
			       (apply (function nconc)
				      (mapcar (function 
					       (lambda (x)
						 (radical-break-string (cdr x))))
					      stroke2))))))
		(if quit-flag
		    (progn
		      (setq quit-flag nil)
		      (setq loop3 t))
		  (insert val2)))))))))

(defun strokes-input (code)
  (interactive (list (if (or current-prefix-arg (null radical-code))
			 (read-radical-code)
		       radical-code)))
  (setq radical-code code)
  (let* ((radical-data (get-radical-data code))
	 (strokes-table (nth 2 radical-data))
	 (loop t)
	 (stroke nil)
	 (nstrokes (1- (length strokes-table)))
	 (val)
	 (query (nth 3 radical-data))
	 (q-chinese-char (nth 0 query))
	 (q-stroke (nth 5 query)))
    (while loop
      (setq stroke (string-to-int
		  (read-from-minibuffer (format "%s(1-%d): " q-stroke nstrokes)
					(if stroke (int-to-string stroke)))))
      (if (not (and (< 0 stroke) (<= stroke nstrokes))) ; 92.9.30 by K.Mitani
	  (setq loop nil)
	(let ((inhibit-quit t))
	  (setq val 
		(menu:select-from-menu
		 (list 'menu (concat q-chinese-char ":")
		       (radical-break-string (aref strokes-table stroke)))))
	  (if quit-flag
	      (setq quit-flag nil)
	    (insert val)
	    (setq loop nil)))))))

(defun radical-break-string (str)
  (let ((len (length str))
	(i 0) j l)
    (while (< i len)
      (setq j (char-bytes (sref str i)))
      (setq l (cons (substring str i (+ i j)) l))
      (setq i (+ i j)))
    (nreverse l)))

;;; radical.el ends here
